/*
	Copyright (C) 2004,2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#ifndef	__LIBMG_H__
#define __LIBMG_H__	1

#ifndef __errno__
extern	int	errno;
#define __errno__	1
#endif

#include	<assert.h>
#include	<dirent.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<stdarg.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<sys/time.h>
#include	<sys/types.h>
#include	<sys/wait.h>
#include	<sysexits.h>
#include	<time.h>
#include	<unistd.h>

#include	<sys/resource.h>

#define	KEY_A		65
#define	KEY_a		97
#define	CLASS_TYPE_DB	1
#define	CLASS_TYPE_DIR	2
#define	CLASS_TYPE_MEM	3
#define	CR		13
#define	DEBUG_MGdb	0
#define	DEBUG_MGdir	0
#define	KEY_ENTER	10
#define	KEY_N		78
#define	KEY_SPACE	32
#define	KEY_X		88
#define	KEY_Y		89
#define	KEY_n		110
#define	KEY_x		120
#define	LINEFEED	10
#define	MAXENTRIES	4096	/* max directory records for MGdirCreate */
#define	MAXPARAMETERS	256
#define	SPACE		32
#define	TAB		9
#define KEY_y		121

#define	SINGLE_LINES	"------------------------------------------------------------------------"
#define	DOUBLE_LINES	"========================================================================"

			char**		MGm__command;
			int		MGm__forkStatus;
			pid_t		MGm__pid;
	struct		stat		fstatBuf;
	struct		stat		lstatBuf;
	unsigned	int		MGm__bufferSize;
	unsigned	int		MGm__stringSize;
typedef	struct 		classDb		classDb; 
typedef	struct 		classDir	classDir; 

#define	MAXSTRINGSIZE	0x1ff
#define	MAXBUFFERSIZE	0xffff

/***********************************
MACROS
************************************/
#define MGmSetString( var, string )				\
	errorCode	= 0;					\
	var	= (char*)calloc( MGrStrlen( string ) + 1, 1 );	\
	if( !var )						\
	{							\
		errorCode	= 1;				\
	}							\
	else							\
	{							\
		MGmStrcat( var, string );	\
	}

#define	MGmStrcpy( buffer, string )	\
	MGm__stringSize	= MGrStrlen( string );	\
	if( MGm__stringSize + 1 > MAXSTRINGSIZE )	\
	{	\
		fprintf( stderr, "%s error: string size exceeds MAXSTRINGSIZE-=>%d\n", id, MAXSTRINGSIZE );	\
		while( fflush( stderr ) );	\
		assert( 0 );	\
	}	\
	strlcpy( buffer, string, MGm__stringSize + 1 );

#define	MGmBuffercpy( buffer, string, size )	\
	MGm__stringSize	= MGrBufferlen( string, size );	\
	if( MGm__stringSize + 1 > size )	\
	{	\
		fprintf( stderr, "%s error: string size exceeds maximum allowed size-=>%d\n", id, size );	\
		while( fflush( stderr ) );	\
		assert( 0 );	\
	}	\
	strlcpy( buffer, string, MGm__stringSize + 1 );

#define	MGmStrcat( buffer, string )	\
	MGm__stringSize	= MGrStrlen( string );	\
	MGm__bufferSize	= MGrStrlen( buffer );	\
	if( MGm__stringSize + MGm__bufferSize + 1 > MAXSTRINGSIZE )	\
	{	\
		fprintf( stderr, "%s error: string size exceeds MAXSTRINGSIZE-=>%d\n", id, MAXSTRINGSIZE );	\
		while( fflush( stderr ) );	\
		assert( 0 );	\
	}	\
	if( strlcat( buffer, string, MGm__stringSize + MGm__bufferSize + 1 ) != MGm__stringSize + MGm__bufferSize )	\
	{	\
		fprintf( stderr, "%s error: string truncated?\n", id );	\
		while( fflush( stderr ) );	\
		assert( 0 );	\
	}

#define	MGmBuffercat( buffer, string, size )	\
	MGm__stringSize	= MGrStrlen( string );	\
	MGm__bufferSize	= MGrStrlen( buffer );	\
	if( MGm__stringSize + MGm__bufferSize + 1 > size )	\
	{	\
		fprintf( stderr, "%s error: string size exceeds max size-=>%d\n", id, size );	\
		while( fflush( stderr ) );	\
		assert( 0 );	\
	}	\
	if( strlcat( buffer, string, MGm__stringSize + MGm__bufferSize + 1 ) != MGm__stringSize + MGm__bufferSize )	\
	{	\
		fprintf( stderr, "%s error: string truncated?\n", id );	\
		while( fflush( stderr ) );	\
		assert( 0 );	\
	}

#define	MGmSystem( command, environment )				\
		MGm__pid	= fork();				\
		if( MGm__pid == 0 )		 /* child */		\
		{							\
			execve( command[0], command, environment );	\
			_exit(127);					\
		}							\
		while( waitpid( MGm__pid, &MGm__forkStatus, 0 ) < 0 )	/* parent */	\
		{							\
			if( WIFEXITED( MGm__forkStatus ) == 0 )		\
			{						\
				fprintf( stderr, "%s error: command %s exited abnormally\n", id, command[0] );	\
				while( fflush( stderr ) );		\
				assert( 0 );				\
			}						\
		}
	

#define	MGmFopen( fileStream, fileName, mode )	\
	if( mode[0] == 'w' )				\
	{						\
/* fprintf( stderr, "%s MGmFileOpen DEBUG m01: fileName-=>%s\n", id, fileName ); */ 	\
/* while( fflush( stderr ) ); */ 			\
		MGm__command	= calloc( sizeof( char** ) * 3, 1 ); \
		MGm__command[0]	= calloc( MAXSTRINGSIZE, 1 );		\
		MGm__command[1]	= calloc( MAXSTRINGSIZE, 1 );		\
		MGm__command[2]	= calloc( MAXSTRINGSIZE, 1 );		\
		MGmStrcpy( MGm__command[0], "/usr/bin/touch" );	\
		MGmStrcpy( MGm__command[1], fileName );		\
		MGm__command[2]	= 0;				\
		MGm__pid	= fork();			\
		if( MGm__pid == 0 )		 /* child */	\
		{					\
/* fprintf( stderr, "%s MGmFileOpen DEBUG m01: executing %s %s\n", id, MGm__command[0], MGm__command[1] ); */ 	\
/* while( fflush( stderr ) ); */ 			\
			execve( MGm__command[0], MGm__command, NULL );	\
			_exit(127); /* don't use exit(); it flushes I/O buffers */	\
		}					\
		while( waitpid( MGm__pid, &MGm__forkStatus, 0 ) < 0 )		/* parent */	\
		{					\
			if( WIFEXITED( MGm__forkStatus ) == 0 )	\
			{				\
				fprintf( stderr, "%s error: command %s %s exited abnormally\n", id, MGm__command[0], MGm__command[1] );	\
				while( fflush( stderr ) );	\
				assert( 0 );		\
			}				\
		}					\
		free( MGm__command[0] );				\
		free( MGm__command[1] );				\
		free( MGm__command[2] );				\
		free( MGm__command );				\
	}						\
	if( lstat( fileName, &lstatBuf ) != 0 )		\
	{						\
		fprintf( stderr, "%s error, lstat of %s failed\n", id, fileName );	\
		perror( "lstat" );			\
		while( fflush( stderr ) );		\
		assert( 0 );				\
	}						\
	if( S_ISLNK( lstatBuf.st_mode ) )		\
	{						\
		fprintf( stderr, "%s error: %s is a symbolic link\n", id, fileName );	\
		while( fflush( stderr ) );		\
		assert( 0 );				\
	}						\
	if( ( fileStream = fopen( fileName, mode ) ) == NULL )	/* Flawfinder: ignore */	\
	{						\
		fprintf( stderr, "%s error: unable to fopen %s in mode %s\n", id, fileName, mode );	\
		perror( "fopen" );			\
		while( fflush( stderr ) );		\
		assert( 0 );				\
	}						\
	if( ( fstat( fileno( fileStream ), &fstatBuf ) ) != 0 )	\
	{						\
		fprintf( stderr, "%s error: cannot fstat %s \n", id, fileName );	\
		perror( "fstat" );			\
		while( fflush( stderr ) );		\
		assert( 0 );				\
	}						\
	if( ( ( ( lstatBuf.st_ino ) != ( fstatBuf.st_ino ) ) || ( ( lstatBuf.st_dev ) != ( fstatBuf.st_dev ) ) ) )	\
	{						\
		fprintf( stderr, "%s error: %s attribute inconsistency\n", id, fileName );	\
		while( fflush( stderr ) );		\
		assert( 0 );				\
	}

char*			MGdbGet(classDb* db, int recordIdx, char* fieldName );
char*			MGdbSeek( classDb* db, char* fieldName, char* keyValue, char* returnField, char* exact );
char*			MGdirGet(classDir* dir);
char*			MGrIntToString( int );
classDb*		MGdbCreate( char* dbFileName, ... );
classDb*		MGdbDelete(classDb* db, int recordIdx );
classDb*		MGdbOpen( char* dbFileName );	
classDb*		MGdbUpDate( char* dbFileName, ... );
classDb*		MGxTermBarCreate( char* title, ... );
classDb*		MGxTermBarDestroy( char* dbFileName, ... );
classDb*		MGxTermBarUpdate( char* dbFileName, ... );
classDir*		MGdirCreate( int mode, char* path );
int			MGdbAdd(classDb* db, ...);
int			MGdbDestroy(classDb* db);
int			MGdbGetRecno(classDb* db );
int			MGdbGetRecordQty(classDb* db);
int			MGdbGoTop(classDb* db);
int			MGdbRead( classDb* db );
int			MGdirDestroy(classDir* dir);
int			MGdirSkip(classDir* dir);
int			MGdirTest(classDir* dir);
int			MGrFileSize( char* );
int			MGrIfDirEntry( char* dirName, char* fileName );
int			MGrIfFileExist( char* );
int			MGrInStringSwap( char* originalString, char* oldString, char* newString );
int			MGrStripComment( char*, char );
unsigned	int	MGrBufferlen( char* buffer, unsigned int maxBufferSize );
unsigned	int	MGrStrlen( char* String );
unsigned long	int	MGrFileTime( char* );

#endif
